;Wrappers functions provided for easy access to the Windows Registry
;Author: Shantanu Gadgil
;If you include this in your project,
;you will need to link against the following:
;advapi32.lib	-> for the actual Reg... APIs
;shwapi.lib	-> for the SHDeleteKey API
;- Enjoy!

.386
.model flat, stdcall  ; 32 bit memory model
option casemap :none  ; case sensitive

include windows.inc

include user32.inc
include kernel32.inc
include advapi32.inc
include shlwapi.inc

include Registry.inc	;for the EQUs

.const
;stuff ONLY required here !!!
;NOT to be exposed via the .INC file :)
szREG_SZ			db	'REG_SZ',0
szREG_DWORD			db	'REG_DWORD',0

.code
Registry_DeleteKey	proc	hKeyRoot:HKEY, pszSubKey:LPCTSTR, pszKeyName:LPCTSTR
	LOCAL	hKey:HKEY
	LOCAL	dwReturnValue:DWORD

	mov	dwReturnValue, REGISTRY_OPERATION_SUCCESS

	;try to open key
	invoke	RegOpenKeyEx,hKeyRoot,pszSubKey,0,KEY_WRITE,addr hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_OPEN_FAILED
		jmp	@f
	.endif
	
	invoke	SHDeleteKey,hKey,pszKeyName
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_DELETE_FAILED
		jmp	@f
	.endif

	;we know what this is :)
	invoke	RegCloseKey,hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CLOSE_FAILED
		jmp	@f
	.endif

@@:
	mov	eax,dwReturnValue
	ret
Registry_DeleteKey endp
 
Registry_DeleteValue	proc	hKeyRoot:HKEY, pszSubKey:LPCTSTR, pszKeyName:LPCTSTR
	LOCAL	hKey:HKEY
	LOCAL	dwReturnValue:DWORD

	mov	dwReturnValue, REGISTRY_OPERATION_SUCCESS
	
	;try to open key
	invoke	RegOpenKeyEx,hKeyRoot,pszSubKey,0,KEY_WRITE,addr hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_OPEN_FAILED
		jmp @f
	.endif
	
	invoke	RegDeleteValue,hKey,pszKeyName
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_DELETE_FAILED
		jmp	@f
	.endif
	
	;we know what this is :)
	invoke	RegCloseKey,hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CLOSE_FAILED
		jmp	@f
	.endif
@@:
	mov	eax, dwReturnValue
	ret
Registry_DeleteValue endp

;dwDataType is one of: REG_DWORD, REG_SZ, REG_EXPAND_SZ, etc.
;It has been tested for the above, should work for others too!!! :)
Registry_ReadValue	proc	hKeyRoot:HKEY,pszSubKey:LPCTSTR,pszKeyName:LPCTSTR,pTarget:LPBYTE,dwDataType:DWORD
	LOCAL	hKey:HKEY
	LOCAL	dwReturnType:DWORD
	LOCAL	dwSize:DWORD
	LOCAL	szExpandBuffer[MAX_SZEXPAND_LENGTH]:BYTE
	LOCAL	dwReturnValue:DWORD
	
	push	esi

	mov	dwReturnValue, REGISTRY_OPERATION_SUCCESS

	;try to open key
	invoke	RegOpenKeyEx,hKeyRoot,pszSubKey,0,KEY_READ,addr hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_OPEN_FAILED
		jmp	@f
	.endif
	
	mov	esi,pTarget
	mov	byte ptr [esi],0
	
	;try to understand datatype of pszKeyValue
	invoke	RegQueryValueEx,hKey,pszKeyName,0,addr dwReturnType,0,addr dwSize
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_QUERY_INITIAL_FAILED
		jmp	@f
	.endif
	
	;check for datatype mismatch
	mov	eax,dwReturnType
	.if	eax != dwDataType
		mov	dwReturnValue, REGISTRY_MISMATCH_FAILED
		jmp	@f
	.endif
	
	;read actual data
	invoke	RegQueryValueEx,hKey,pszKeyName,0,addr dwReturnType,pTarget,addr dwSize
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_QUERY_ACTUAL_FAILED
		jmp	@f
	.endif
	
	;special case
	.if	dwDataType == REG_EXPAND_SZ
		invoke	ExpandEnvironmentStrings,addr szExpandBuffer,pTarget,MAX_SZEXPAND_LENGTH
		
		invoke	lstrcpy,pTarget,addr szExpandBuffer
	.endif
	
	;we know what this is :)
	invoke	RegCloseKey,hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CLOSE_FAILED
		jmp	@f
	.endif
@@:
	pop	esi

	mov	eax, dwReturnValue
	ret
Registry_ReadValue	endp

Registry_WriteString	proc	hKeyRoot:HKEY, pszSubKey:LPCTSTR, pszKeyName:LPCTSTR, pszValue:LPCTSTR
	LOCAL	hKey:HKEY
	LOCAL	dwReturnValue:DWORD

	push	esi
	
	mov	dwReturnValue, REGISTRY_OPERATION_SUCCESS

	invoke	RegCreateKeyEx,hKeyRoot,pszSubKey,0,offset szREG_SZ,0,KEY_WRITE,0,addr hKey,0
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CREATE_FAILED
		jmp	@f
	.endif
	
	xor	eax,eax
	mov	esi,pszValue
	.while	byte ptr [esi] != 0
		inc	esi
		inc	eax
	.endw
	inc	eax	;NULL terminator
	
	invoke	RegSetValueEx,hKey,pszKeyName,0,REG_SZ,pszValue,eax
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_SETVALUE_FAILED
		jmp	@f
	.endif
	
	invoke	RegCloseKey,hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CLOSE_FAILED
		jmp	@f
	.endif
@@:
	pop	esi
	
	mov	eax, dwReturnValue
	ret
Registry_WriteString	endp

Registry_WriteDword	proc	hKeyRoot:HKEY, pszSubKey:LPCTSTR, pszKeyName:LPCTSTR, dwValue:DWORD
	LOCAL	hKey:HKEY
	LOCAL	dwReturnValue:DWORD
	
	mov	dwReturnValue, REGISTRY_OPERATION_SUCCESS

	invoke	RegCreateKeyEx,hKeyRoot,pszSubKey,0,offset szREG_DWORD,0,KEY_WRITE,0,addr hKey,0
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CREATE_FAILED
		jmp	@f
	.endif
	
	invoke	RegSetValueEx,hKey,pszKeyName,0,REG_DWORD,addr dwValue,sizeof DWORD
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_SETVALUE_FAILED
		jmp	@f
	.endif
	
	invoke	RegCloseKey,hKey
	.if	eax != ERROR_SUCCESS
		mov	dwReturnValue, REGISTRY_CLOSE_FAILED
		jmp	@f
	.endif

@@:
	mov	eax, dwReturnValue
	ret
Registry_WriteDword endp

end
